package me.icymint.libra.sage.core.query;

import java.sql.BatchUpdateException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Arrays;

import me.icymint.libra.sage.core.param.ParamHandler;
import me.icymint.libra.sage.core.result.ResultHandler;

/**
 * 抽象执行父类。
 * 
 * @author Daniel
 * @datetime 2013-1-4上午11:19:28
 * 
 * @param <S>
 */
public abstract class AbstractExecute<S> extends
		AbstractSqlQuery<Void, PreparedStatement, S, Void> {

	/**
	 * 执行参数列表。
	 */
	private Object[][] params;

	protected AbstractExecute(S sql, int[] types, Object[]... params) {
		super(sql, null, 1, types);
		this.params = params;
	}

	/**
	 * 获取预处理语句。
	 */
	@Override
	protected PreparedStatement getStatement(Connection conn, String sql)
			throws SQLException {
		return conn.prepareStatement(sql);
	}

	/**
	 * 结果处理，直接返回原值。参数{@link ResultHandler}无效。
	 */
	@Override
	protected Void handle(ResultHandler<Void> rh, Void v) throws SQLException {
		return v;
	}

	/**
	 * 执行，分为批处理执行和单个执行，判断条件是是否有参数。
	 */
	@Override
	protected Void query(PreparedStatement pps,
			ParamHandler<PreparedStatement> ph) throws SQLException {
		if (params.length > 0) {
			for (Object[] o : params) {
				ph.handle(pps, getTypes(), o).addBatch();
			}
			try {
				pps.executeBatch();
			} catch (BatchUpdateException e) {
				int[] x = e.getUpdateCounts();
				for (int i = 0; i < x.length; i++) {
					if (x[i] == Statement.EXECUTE_FAILED) {
						throw new SQLException(Arrays.toString(params[i]), e);
					}
				}
				throw e;
			}
		} else {
			pps.executeUpdate();
		}
		return null;
	}
}
