package me.icymint.libra.sage.model;

import me.icymint.libra.sage.core.JdbcUtils;
import me.icymint.libra.sage.mask.DefaultMask;
import me.icymint.libra.sage.mask.Mask;
import me.icymint.libra.sage.model.object.SqlObject;

/**
 * 该类集成了jdbc操作，以及SQL工厂类实例的一些基本信息。
 * 
 * @author Daniel
 * @datetime 2012-11-12上午11:57:07
 * 
 */
public class SqlInfo implements SqlObject {

	private final Mask mask;
	private final String prefix;
	private final boolean ismask;
	private final JdbcUtils su;
	private boolean all = false;

	public SqlInfo(boolean ismask, JdbcUtils su) {
		this(null, null, ismask, su);
	}

	/**
	 * 构建SqlInfo工具。
	 * 
	 * @param prefix
	 * @param mask
	 * @param ismask
	 * @param su
	 */
	public SqlInfo(String prefix, Mask mask, boolean ismask, JdbcUtils su) {
		if (mask == null)
			mask = new DefaultMask();
		this.mask = mask;
		if (prefix == null || !prefix.matches("[_a-zA-Z][_0-9a-zA-Z]{0,14}"))
			prefix = "";
		this.prefix = prefix;
		this.ismask = ismask;
		if (su == null)
			throw new NullPointerException("数据库操作类SageUtils为空！");
		this.su = su;
	}

	@Override
	public String getId() {
		return su.getDefaultCatalog();
	}

	/**
	 * 获取Jdbc操作方法。
	 * 
	 * @return
	 */
	public JdbcUtils getJdbcUtils() {
		return su;
	}

	/**
	 * 屏蔽类。
	 */
	public Mask getMask() {
		return mask;
	}

	/**
	 * 数据库名称。
	 */
	@Override
	public String getName() {
		return su.getDefaultCatalog();
	}

	/**
	 * 数据库中所有名称的前缀。
	 */
	@Override
	public String getPrefix() {
		return prefix;
	}

	@Override
	public SqlInfo getSqlInfo() {
		return this;
	}

	/**
	 * 是否被屏蔽。
	 */
	@Override
	public boolean isMask() {
		return ismask;
	}

	/**
	 * 是否支持所有数据库。
	 */
	public boolean isSupportAllDatabases() {
		return all;
	}

	/**
	 * 设置是否支持所有数据库。
	 * 
	 * @param issupport
	 */
	public void setAsSupportAllDatabases(boolean issupport) {
		this.all = issupport;
	}

}
