package me.icymint.libra.sage.model.format;

import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import me.icymint.libra.sage.model.SqlFunction;
import me.icymint.libra.sage.model.SqlTable;
import me.icymint.libra.sage.model.SqlVar;
import me.icymint.libra.sage.model.operator.FunctionManager;
import me.icymint.libra.sage.model.operator.SQLNotExistsException;
import me.icymint.libra.sage.model.operator.TableManager;
import me.icymint.libra.sage.model.operator.VarManager;

/**
 * 把Sql代码中的数字指标转换成名称指标。 例如：$t(0)表示第1张表table1，经过格式化以后，其变成$t(table1)。
 * 注意，数字指标从0开始计数。
 * 
 * @author Daniel
 * @datetime 2012-11-14上午9:08:56
 * 
 */
public class NumberToNameFormat extends AbstractFormat {

	private static PatternFormat[] get(Object obj) {
		List<PatternFormat> pf = new LinkedList<PatternFormat>();
		if (obj instanceof VarManager) {
			pf.add(getVarFormat((VarManager) obj));
		}
		if (obj instanceof TableManager) {
			pf.add(getTableFormat((TableManager) obj));
			pf.add(getTableVarFormat((TableManager) obj));
		}
		if (obj instanceof FunctionManager) {
			pf.add(getFunctionFormat((FunctionManager) obj));
		}
		return pf.toArray(new PatternFormat[] {});

	}

	public static PatternFormat getFunctionFormat(final FunctionManager fm) {
		return new PatternFormat(Pattern.compile("\\$f\\(" + ID + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				SqlFunction v = null;
				try {
					v = fm.getFunction(Integer.valueOf(m.group(1)));
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"函数"), e);
				}
				return "$f(" + v.getName() + ")";
			}

		};
	}

	public static PatternFormat getTableFormat(final TableManager tm) {
		return new PatternFormat(Pattern.compile("\\$t\\(" + ID + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				SqlTable v = null;
				try {
					v = tm.getTable(Integer.valueOf(m.group(1)));
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"表格"), e);
				}
				return "$t(" + v.getName() + ")";
			}

		};
	}

	public static PatternFormat getTableVarFormat(final TableManager tm) {
		return new PatternFormat(Pattern.compile("\\$t\\(" + ID + "," + ID
				+ "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {

				SqlTable t = null;
				SqlVar v = null;
				try {
					t = tm.getTable(Integer.valueOf(m.group(1)));
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"表格"), e);
				}

				try {
					v = t.getVar(Integer.valueOf(m.group(2)));
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"字段"), e);
				}

				return "$t(" + t.getName() + "," + v.getName() + ")";
			}

		};
	}

	public static PatternFormat getVarFormat(final VarManager vm) {
		return new PatternFormat(Pattern.compile("\\$v\\(" + ID + "\\)")) {

			@Override
			public String format(String sql, Matcher m)
					throws SqlFormatException {
				SqlVar v = null;
				try {
					v = vm.getVar(Integer.valueOf(m.group(1)));
				} catch (SQLNotExistsException e) {
					throw new SqlFormatException(errorMessage(sql, m.group(),
							"字段"), e);
				}
				return "$v(" + v.getName() + ")";
			}

		};
	}

	public NumberToNameFormat(Object obj) {
		this(get(obj));
	}

	public NumberToNameFormat(PatternFormat... pf) {
		super(pf);
	}
}
