package me.icymint.libra.sage.model.object;

import me.icymint.libra.sage.core.JdbcUtils;
import me.icymint.libra.sage.core.dialect.SqlDialect;
import me.icymint.libra.sage.model.SqlInfo;

import org.apache.log4j.Logger;

/**
 * 抽象SQL对象，所有SQL对象的父类。
 * 
 * @author Daniel
 * @datetime 2012-12-19下午2:32:33
 * 
 */
public abstract class AbstractSql implements SqlObject {

	/**
	 * 日志记录。
	 */
	protected Logger logger = Logger.getLogger(getClass());
	/**
	 * SQL对象名称
	 */
	private final String name;
	/**
	 * SQL对象前缀
	 */
	private final String prefix;
	/**
	 * SQL公用信息
	 */
	private final SqlInfo si;
	/**
	 * SQL对象标签
	 */
	private String label = null;
	/**
	 * SQL对象屏蔽状态。
	 */
	private boolean isMask;

	protected AbstractSql(SqlInfo si, String prefix, String name, boolean isMask) {
		this.si = si;
		this.prefix = prefix;
		if (name == null)
			throw new NullPointerException("Sql对象名称为空！");
		this.name = name.toLowerCase();
		this.isMask = isMask;
	}

	/**
	 * 获取描述信息，名称和标签的合集。 标签为空，返回名称 不为空，则返回名称和标签的组合。<br>
	 * 例如：SQL对象 abc，其标签为 “字母”，那么返回值为 “abc(字母)”
	 */
	public String getDescription() {
		if (label == null || label.equals("")) {
			return getName();
		} else {
			return getName() + "(" + label + ")";
		}
	}

	/**
	 * 
	 * @return 获取SQL方言
	 */
	public SqlDialect getDialect() {
		return this.getJdbcUtils().getDialect();
	}

	@Override
	public String getId() {
		return getSqlObjectType().name() + "." + name;
	}

	/**
	 * 
	 * @return 获取JDBC操作对象。
	 */
	public JdbcUtils getJdbcUtils() {
		return si != null ? si.getJdbcUtils() : null;
	}

	/**
	 * 
	 * @return 获取标签
	 */
	public String getLabel() {
		return label == null ? getName() : label;
	}

	/**
	 * 
	 * @return 获取该对象的是否屏蔽。
	 */
	public boolean getMask() {
		return isMask;
	}

	@Override
	public String getName() {
		return name;
	}

	@Override
	public String getPrefix() {
		return prefix == null ? si.getPrefix() : prefix;
	}

	@Override
	public SqlInfo getSqlInfo() {
		return si;
	}

	/**
	 * @see SqlObjectType
	 * @return 获取当前SQL对象的类型
	 */
	protected abstract SqlObjectType getSqlObjectType();

	@Override
	public boolean isMask() {
		return isMask ? si.isMask() : false;
	}

	/**
	 * 设置标签。
	 * 
	 * @param label
	 */
	public void setLabel(String label) {
		this.label = label;
	}

	/**
	 * 设置是否屏蔽。
	 * 
	 * @param ismask
	 */
	public void setMask(boolean ismask) {
		this.isMask = ismask;
	}

	@Override
	public String toString() {
		if (!this.isMask())
			return getName();
		String pre = getPrefix();
		if (pre.equals(""))
			return si.getMask().mask(getName());
		else
			return pre + "_" + si.getMask().mask(getName());
	}

}
