package me.icymint.libra.sage.model.object;

import me.icymint.libra.sage.model.SqlInfo;
import me.icymint.libra.sage.model.SqlVar;
import me.icymint.libra.sage.model.operator.ManagerObjects;
import me.icymint.libra.sage.model.operator.SQLExistsException;
import me.icymint.libra.sage.model.operator.SQLNotExistsException;
import me.icymint.libra.sage.model.operator.VarManager;

/**
 * SQL Var管理器。
 * 
 * @author Daniel
 * @datetime 2013-1-4下午2:13:31
 * 
 */
public abstract class VarManagerObject extends AbstractSql implements
		VarManager {

	private final ManagerObjects<SqlVar> mo;

	protected VarManagerObject(SqlInfo si, String prefix, String name,
			boolean isMask, SqlVar... sv) {
		super(si, prefix, name, isMask);
		mo = new ManagerObjects<SqlVar>(sv, SqlVar.class) {

			@Override
			protected String getName(SqlVar t) {
				return getVarId(t);
			}
		};
	}

	@Override
	public void addVar(SqlVar v) throws SQLExistsException {
		mo.add(v);
	}

	@Override
	public void delVar(SqlVar v) {
		mo.del(v);
	}

	@Override
	public SqlVar getVar(int ind) throws SQLNotExistsException {
		return mo.get(ind);
	}

	@Override
	public SqlVar getVar(String name) throws SQLNotExistsException {
		return mo.get(name);
	}

	/**
	 * 表示用于管理的SqlVar以何种方式来管理。
	 * 
	 * @param v
	 */
	protected String getVarId(SqlVar v) {
		return v.getName();
	}

	@Override
	public int getVarIndex(String name) throws SQLNotExistsException {
		return mo.getIndexOf(name);
	}

	@Override
	public SqlVar[] getVars() {
		return mo.getAll();
	}

	@Override
	public int getVarSize() {
		return mo.getSize();
	}

}
